@@ -47,6 +47,10 @@ typedef struct
 	GtkWidget	*spinner;
 	GtkWidget	*label;
 	GtkWidget	*checkbox;
+	GtkWidget	*label_tag_foreign;
+	GtkWidget	*label_tag_warning;
+	GtkWidget	*label_tag_webapp;
+	GtkWidget	*label_tag_nonfree;
 	gboolean	 colorful;
 	gboolean	 show_codec;
 	gboolean	 show_update;
@@ -83,7 +87,8 @@ gs_app_row_get_description (GsAppRow *app_row)
 
 	/* convert the markdown update description into PangoMarkup */
 	if (priv->show_update &&
-	    gs_app_get_state (priv->app) == AS_APP_STATE_UPDATABLE) {
+	    (gs_app_get_state (priv->app) == AS_APP_STATE_UPDATABLE ||
+	     gs_app_get_state (priv->app) == AS_APP_STATE_UPDATABLE_LIVE)) {
 		tmp = gs_app_get_update_details (priv->app);
 		if (tmp != NULL && tmp[0] != '\0') {
 			g_autoptr(GsMarkdown) markdown = NULL;
@@ -135,16 +140,41 @@ gs_app_row_refresh (GsAppRow *app_row)
 		break;
 	}
 
-	/* join the lines*/
+	/* join the description lines */
 	str = gs_app_row_get_description (app_row);
 	gs_string_replace (str, "\n", " ");
 	gtk_label_set_markup (GTK_LABEL (priv->description_label), str->str);
 	g_string_free (str, TRUE);
 
+	/* add warning */
+	if (gs_app_get_kind (priv->app) == GS_APP_KIND_FIRMWARE_UPDATE) {
+		gtk_label_set_text (GTK_LABEL (priv->label_tag_warning),
+				    /* TRANSLATORS: during the update the device
+				     * will restart into a special update-only mode */
+				    _("Device cannot be used during update."));
+		gtk_widget_show (priv->label_tag_warning);
+	}
+
+	/* add tags */
+	gtk_widget_set_visible (priv->label_tag_webapp,
+				gs_app_get_id_kind (priv->app) == AS_ID_KIND_WEB_APP);
+	switch (gs_app_get_id_kind (priv->app)) {
+	case AS_ID_KIND_WEB_APP:
+	case AS_ID_KIND_UNKNOWN:
+		break;
+	default:
+		gtk_widget_set_visible (priv->label_tag_nonfree,
+					!gs_app_get_licence_is_free (priv->app));
+		gtk_widget_set_visible (priv->label_tag_foreign,
+					!gs_app_get_provenance (priv->app));
+		break;
+	}
+
 	gtk_label_set_label (GTK_LABEL (priv->name_label),
 			     gs_app_get_name (priv->app));
 	if (priv->show_update &&
-	    gs_app_get_state (priv->app) == AS_APP_STATE_UPDATABLE) {
+	    (gs_app_get_state (priv->app) == AS_APP_STATE_UPDATABLE ||
+	     gs_app_get_state (priv->app) == AS_APP_STATE_UPDATABLE_LIVE)) {
 		gtk_widget_show (priv->version_label);
 		gtk_widget_hide (priv->star);
 		gtk_label_set_label (GTK_LABEL (priv->version_label),
@@ -232,6 +262,18 @@ gs_app_row_refresh (GsAppRow *app_row)
 		 * allows the application to be easily installed */
 		gtk_button_set_label (GTK_BUTTON (priv->button), _("Install"));
 		break;
+	case AS_APP_STATE_UPDATABLE_LIVE:
+		gtk_widget_set_visible (priv->button, TRUE);
+		if (priv->show_update) {
+			/* TRANSLATORS: this is a button in the updates panel
+			 * that allows the app to be easily updated live */
+			gtk_button_set_label (GTK_BUTTON (priv->button), _("Install"));
+		} else {
+			/* TRANSLATORS: this is a button next to the search
+			 * results that allows the app to be easily updated live */
+			gtk_button_set_label (GTK_BUTTON (priv->button), _("Update"));
+		}
+		break;
 	case AS_APP_STATE_UPDATABLE:
 	case AS_APP_STATE_INSTALLED:
 		if (gs_app_get_kind (priv->app) != GS_APP_KIND_SYSTEM)
@@ -262,10 +304,20 @@ gs_app_row_refresh (GsAppRow *app_row)
 		break;
 	}
 
-	gtk_widget_set_visible (priv->button_box, !priv->show_update);
+	/* hide buttons in the update list, unless the app is live updatable */
+	switch (gs_app_get_state (priv->app)) {
+	case AS_APP_STATE_UPDATABLE_LIVE:
+	case AS_APP_STATE_INSTALLING:
+		gtk_widget_set_visible (priv->button_box, TRUE);
+		break;
+	default:
+		gtk_widget_set_visible (priv->button_box, !priv->show_update);
+		break;
+	}
 
 	if (priv->selectable) {
 		if (gs_app_get_id_kind (priv->app) == AS_ID_KIND_DESKTOP ||
+		    gs_app_get_id_kind (priv->app) == AS_ID_KIND_RUNTIME ||
 		    gs_app_get_id_kind (priv->app) == AS_ID_KIND_WEB_APP)
 			gtk_widget_set_visible (priv->checkbox, TRUE);
 		gtk_widget_set_sensitive (priv->button, FALSE);
@@ -463,6 +515,10 @@ gs_app_row_class_init (GsAppRowClass *klass)
 	gtk_widget_class_bind_template_child_private (widget_class, GsAppRow, spinner);
 	gtk_widget_class_bind_template_child_private (widget_class, GsAppRow, label);
 	gtk_widget_class_bind_template_child_private (widget_class, GsAppRow, checkbox);
+	gtk_widget_class_bind_template_child_private (widget_class, GsAppRow, label_tag_warning);
+	gtk_widget_class_bind_template_child_private (widget_class, GsAppRow, label_tag_foreign);
+	gtk_widget_class_bind_template_child_private (widget_class, GsAppRow, label_tag_webapp);
+	gtk_widget_class_bind_template_child_private (widget_class, GsAppRow, label_tag_nonfree);
 }
 
 static void
