@@ -73,11 +73,13 @@ struct _GsApp
 	GPtrArray		*keywords;
 	GHashTable		*urls;
 	gchar			*licence;
+	GsAppQuality		 licence_quality;
 	gchar			**menu_path;
 	gchar			*origin;
 	gchar			*update_version;
 	gchar			*update_version_ui;
 	gchar			*update_details;
+	AsUrgencyKind		 update_urgency;
 	gchar			*management_plugin;
 	gint			 rating;
 	gint			 rating_confidence;
@@ -98,7 +100,9 @@ struct _GsApp
 	guint64			 install_date;
 	guint64			 kudos;
 	gboolean		 to_be_installed;
+	gboolean		 provenance;
 	AsBundle		*bundle;
+	gboolean		 licence_is_free;
 };
 
 enum {
@@ -153,6 +157,10 @@ gs_app_kind_to_string (GsAppKind kind)
 		return "source";
 	if (kind == GS_APP_KIND_CORE)
 		return "core";
+	if (kind == GS_APP_KIND_DISTRO_UPGRADE)
+		return "distro-upgrade";
+	if (kind == GS_APP_KIND_FIRMWARE_UPDATE)
+		return "firmware-update";
 	return NULL;
 }
 
@@ -213,8 +221,8 @@ gs_app_to_string (GsApp *app)
 		g_string_append (str, "\tkudo:\tperfect-screenshots\n");
 	if ((app->kudos & GS_APP_KUDO_HIGH_CONTRAST) > 0)
 		g_string_append (str, "\tkudo:\thigh-contrast\n");
-	if ((app->kudos & GS_APP_KUDO_APPDATA_DESCRIPTION) > 0)
-		g_string_append (str, "\tkudo:\tappdata-description\n");
+	if ((app->kudos & GS_APP_KUDO_HI_DPI_ICON) > 0)
+		g_string_append (str, "\tkudo:\thi-dpi-icon\n");
 	g_string_append_printf (str, "\tkudo-percentage:\t%i\n",
 				gs_app_get_kudos_percentage (app));
 	if (app->name != NULL)
@@ -244,6 +252,10 @@ gs_app_to_string (GsApp *app)
 		g_string_append_printf (str, "\tupdate-details:\t%s\n",
 					app->update_details);
 	}
+	if (app->update_urgency != AS_URGENCY_KIND_UNKNOWN) {
+		g_string_append_printf (str, "\tupdate-urgency:\t%i\n",
+					app->update_urgency);
+	}
 	if (app->summary != NULL)
 		g_string_append_printf (str, "\tsummary:\t%s\n", app->summary);
 	if (app->description != NULL)
@@ -271,6 +283,8 @@ gs_app_to_string (GsApp *app)
 		g_string_append_printf (str, "\turl{homepage}:\t%s\n", tmp);
 	if (app->licence != NULL)
 		g_string_append_printf (str, "\tlicence:\t%s\n", app->licence);
+	if (app->management_plugin != NULL)
+		g_string_append_printf (str, "\tmanagement-plugin:\t%s\n", app->management_plugin);
 	if (app->summary_missing != NULL)
 		g_string_append_printf (str, "\tsummary-missing:\t%s\n", app->summary_missing);
 	if (app->menu_path != NULL &&
@@ -413,6 +427,7 @@ gs_app_set_state_internal (GsApp *app, AsAppState state)
 		    state == AS_APP_STATE_AVAILABLE ||
 		    state == AS_APP_STATE_AVAILABLE_LOCAL ||
 		    state == AS_APP_STATE_UPDATABLE ||
+		    state == AS_APP_STATE_UPDATABLE_LIVE ||
 		    state == AS_APP_STATE_UNAVAILABLE)
 			state_change_ok = TRUE;
 		break;
@@ -455,6 +470,12 @@ gs_app_set_state_internal (GsApp *app, AsAppState state)
 		    state == AS_APP_STATE_REMOVING)
 			state_change_ok = TRUE;
 		break;
+	case AS_APP_STATE_UPDATABLE_LIVE:
+		/* updatable-live has to go into an action state */
+		if (state == AS_APP_STATE_UNKNOWN ||
+		    state == AS_APP_STATE_INSTALLING)
+			state_change_ok = TRUE;
+		break;
 	case AS_APP_STATE_UNAVAILABLE:
 		/* updatable has to go into an action state */
 		if (state == AS_APP_STATE_UNKNOWN ||
@@ -1266,18 +1287,56 @@ gs_app_get_licence (GsApp *app)
 	return app->licence;
 }
 
+/**
+ * gs_app_get_licence_is_free:
+ */
+gboolean
+gs_app_get_licence_is_free (GsApp *app)
+{
+	g_return_val_if_fail (GS_IS_APP (app), FALSE);
+	return app->licence_is_free;
+}
+
+/**
+ * gs_app_get_licence_token_is_nonfree:
+ */
+static gboolean
+gs_app_get_licence_token_is_nonfree (const gchar *token)
+{
+	/* grammar */
+	if (g_strcmp0 (token, "(") == 0)
+		return FALSE;
+	if (g_strcmp0 (token, ")") == 0)
+		return FALSE;
+
+	/* a token, but still nonfree */
+	if (g_strcmp0 (token, "@LicenseRef-proprietary") == 0)
+		return TRUE;
+
+	/* if it has a prefix, assume it is free */
+	return token[0] != '@';
+}
+
 /**
  * gs_app_set_licence:
  */
 void
-gs_app_set_licence (GsApp *app, const gchar *licence)
+gs_app_set_licence (GsApp *app, const gchar *licence, GsAppQuality quality)
 {
 	GString *urld;
 	guint i;
 	g_auto(GStrv) tokens = NULL;
 
+	/* only save this if the data is sufficiently high quality */
+	if (quality <= app->licence_quality)
+		return;
+	app->licence_quality = quality;
+
 	g_return_if_fail (GS_IS_APP (app));
 
+	/* assume free software until we find an unmatched SPDX token */
+	app->licence_is_free = TRUE;
+
 	/* tokenize the license string and URLify any SPDX IDs */
 	urld = g_string_sized_new (strlen (licence) + 1);
 	tokens = as_utils_spdx_license_tokenize (licence);
@@ -1301,12 +1360,39 @@ gs_app_set_licence (GsApp *app, const gchar *licence)
 			continue;
 		}
 
+		/* do the best we can */
+		if (gs_app_get_licence_token_is_nonfree (tokens[i])) {
+			g_debug ("nonfree licence from %s: '%s'",
+				 gs_app_get_id (app), tokens[i]);
+			app->licence_is_free = FALSE;
+		}
+
 		/* legacy literal text */
 		if (g_str_has_prefix (tokens[i], "#")) {
 			g_string_append (urld, tokens[i] + 1);
 			continue;
 		}
 
+		/* proprietary software */
+		if (g_strcmp0 (tokens[i], "@LicenseRef-proprietary") == 0) {
+			const gchar *url = "https://en.wikipedia.org/wiki/Proprietary_software";
+			g_string_append_printf (urld,
+						"<a href=\"%s\">%s</a>",
+						/* TRANSLATORS: non-free app */
+						url, _("Proprietary"));
+			continue;
+		}
+
+		/* public domain */
+		if (g_strcmp0 (tokens[i], "@LicenseRef-public-domain") == 0) {
+			const gchar *url = "https://en.wikipedia.org/wiki/Public_domain";
+			g_string_append_printf (urld,
+						"<a href=\"%s\">%s</a>",
+						/* TRANSLATORS: see the wikipedia page */
+						url, _("Public domain"));
+			continue;
+		}
+
 		/* SPDX value */
 		if (g_str_has_prefix (tokens[i], "@")) {
 			g_string_append_printf (urld,
@@ -1497,6 +1583,26 @@ gs_app_set_update_details (GsApp *app, const gchar *update_details)
 	app->update_details = g_strdup (update_details);
 }
 
+/**
+ * gs_app_get_update_urgency:
+ */
+AsUrgencyKind
+gs_app_get_update_urgency (GsApp *app)
+{
+	g_return_val_if_fail (GS_IS_APP (app), AS_URGENCY_KIND_UNKNOWN);
+	return app->update_urgency;
+}
+
+/**
+ * gs_app_set_update_urgency:
+ */
+void
+gs_app_set_update_urgency (GsApp *app, AsUrgencyKind update_urgency)
+{
+	g_return_if_fail (GS_IS_APP (app));
+	app->update_urgency = update_urgency;
+}
+
 /**
  * gs_app_get_management_plugin:
  */
@@ -1904,6 +2010,8 @@ gs_app_get_kudos_percentage (GsApp *app)
 		percentage += 20;
 	if ((app->kudos & GS_APP_KUDO_HIGH_CONTRAST) > 0)
 		percentage += 20;
+	if ((app->kudos & GS_APP_KUDO_HI_DPI_ICON) > 0)
+		percentage += 20;
 
 	/* popular apps should be at *least* 50% */
 	if ((app->kudos & GS_APP_KUDO_POPULAR) > 0)
@@ -1958,6 +2066,24 @@ gs_app_get_bundle (GsApp *app)
 	return app->bundle;
 }
 
+/**
+ * gs_app_get_provenance:
+ */
+gboolean
+gs_app_get_provenance (GsApp *app)
+{
+	return app->provenance;
+}
+
+/**
+ * gs_app_set_provenance:
+ */
+void
+gs_app_set_provenance (GsApp *app, gboolean provenance)
+{
+	app->provenance = provenance;
+}
+
 /**
  * gs_app_subsume:
  *
@@ -2002,6 +2128,8 @@ gs_app_subsume (GsApp *app, GsApp *other)
 		gs_app_set_description (app, other->description_quality, other->description);
 	if (other->update_details != NULL)
 		gs_app_set_update_details (app, other->update_details);
+	if (other->update_urgency != AS_URGENCY_KIND_UNKNOWN)
+		gs_app_set_update_urgency (app, other->update_urgency);
 	if (other->update_version != NULL)
 		gs_app_set_update_version_internal (app, other->update_version);
 	if (other->pixbuf != NULL)
@@ -2080,7 +2208,7 @@ gs_app_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *
 		g_value_set_string (value, app->description);
 		break;
 	case PROP_RATING:
-		g_value_set_uint (value, app->rating);
+		g_value_set_int (value, app->rating);
 		break;
 	case PROP_KIND:
 		g_value_set_uint (value, app->kind);
@@ -2296,6 +2424,9 @@ gs_app_class_init (GsAppClass *klass)
 				   G_PARAM_READWRITE | G_PARAM_CONSTRUCT);
 	g_object_class_install_property (object_class, PROP_PROGRESS, pspec);
 
+	/**
+	 * GsApp:install-date:
+	 */
 	pspec = g_param_spec_uint64 ("install-date", NULL, NULL,
 				     0, G_MAXUINT64, 0,
 				     G_PARAM_READWRITE | G_PARAM_CONSTRUCT);
