@@ -39,13 +39,15 @@ typedef enum {
 
 typedef enum {
 	GS_APP_KIND_UNKNOWN,
-	GS_APP_KIND_NORMAL,	/* app	[ install:1 remove:1 update:1 ] */
-	GS_APP_KIND_SYSTEM,	/* app	[ install:0 remove:0 update:1 ] */
-	GS_APP_KIND_PACKAGE,	/* pkg	[ install:0 remove:0 update:1 ] */
-	GS_APP_KIND_OS_UPDATE,	/* pkg	[ install:0 remove:0 update:1 ] */
-	GS_APP_KIND_MISSING,	/* meta	[ install:0 remove:0 update:0 ] */
-	GS_APP_KIND_SOURCE,	/* src	[ install:1 remove:0 update:0 ] */
-	GS_APP_KIND_CORE,	/* pkg	[ install:0 remove:0 update:1 ] */
+	GS_APP_KIND_NORMAL,		/* app	[ install:1 remove:1 update:1 ] */
+	GS_APP_KIND_SYSTEM,		/* app	[ install:0 remove:0 update:1 ] */
+	GS_APP_KIND_PACKAGE,		/* pkg	[ install:0 remove:0 update:1 ] */
+	GS_APP_KIND_OS_UPDATE,		/* pkg	[ install:0 remove:0 update:1 ] */
+	GS_APP_KIND_MISSING,		/* meta	[ install:0 remove:0 update:0 ] */
+	GS_APP_KIND_SOURCE,		/* src	[ install:1 remove:0 update:0 ] */
+	GS_APP_KIND_CORE,		/* pkg	[ install:0 remove:0 update:1 ] */
+	GS_APP_KIND_DISTRO_UPGRADE,	/* meta	[ install:0 remove:0 update:1 ] */
+	GS_APP_KIND_FIRMWARE_UPDATE,	/* app	[ install:0 remove:0 update:1 ] */
 	GS_APP_KIND_LAST
 } GsAppKind;
 
@@ -72,7 +74,7 @@ typedef enum {
 	GS_APP_KUDO_IBUS_HAS_SYMBOL		= 1 << 11,
 	GS_APP_KUDO_PERFECT_SCREENSHOTS		= 1 << 12,
 	GS_APP_KUDO_HIGH_CONTRAST		= 1 << 13,
-	GS_APP_KUDO_APPDATA_DESCRIPTION		= 1 << 14,
+	GS_APP_KUDO_HI_DPI_ICON			= 1 << 14,
 	GS_APP_KUDO_LAST
 } GsAppKudo;
 
@@ -156,8 +158,10 @@ void		 gs_app_set_url			(GsApp		*app,
 						 AsUrlKind	 kind,
 						 const gchar	*url);
 const gchar	*gs_app_get_licence		(GsApp		*app);
+gboolean	 gs_app_get_licence_is_free	(GsApp		*app);
 void		 gs_app_set_licence		(GsApp		*app,
-						 const gchar	*licence);
+						 const gchar	*licence,
+						 GsAppQuality	 quality);
 gchar		**gs_app_get_menu_path		(GsApp		*app);
 void		 gs_app_set_menu_path		(GsApp		*app,
 						 gchar		**menu_path);
@@ -174,6 +178,9 @@ void		 gs_app_set_update_version	(GsApp		*app,
 const gchar	*gs_app_get_update_details	(GsApp		*app);
 void		 gs_app_set_update_details	(GsApp		*app,
 						 const gchar	*update_details);
+AsUrgencyKind	 gs_app_get_update_urgency	(GsApp		*app);
+void		 gs_app_set_update_urgency	(GsApp		*app,
+						 AsUrgencyKind	 update_urgency);
 const gchar	*gs_app_get_management_plugin	(GsApp		*app);
 void		 gs_app_set_management_plugin	(GsApp		*app,
 						 const gchar	*management_plugin);
@@ -244,6 +251,10 @@ AsBundle	*gs_app_get_bundle		(GsApp		*app);
 void		 gs_app_set_bundle		(GsApp		*app,
 						 AsBundle	*bundle);
 
+gboolean	 gs_app_get_provenance		(GsApp		*app);
+void		 gs_app_set_provenance		(GsApp		*app,
+						 gboolean	 provenance);
+
 G_END_DECLS
 
 #endif /* __GS_APP_H */
