@@ -29,6 +29,7 @@
 #include <gtk/gtk.h>
 #include <gio/gio.h>
 #include <gio/gdesktopappinfo.h>
+#include <libsoup/soup.h>
 #include <packagekit-glib2/packagekit.h>
 
 #ifdef GDK_WINDOWING_X11
@@ -36,13 +37,10 @@
 #endif
 
 #include "gs-dbus-helper.h"
-#include "gs-box.h"
 #include "gs-first-run-dialog.h"
 #include "gs-shell.h"
 #include "gs-update-monitor.h"
 #include "gs-proxy-settings.h"
-#include "gs-plugin-loader.h"
-#include "gs-profile.h"
 #include "gs-shell-search-provider.h"
 #include "gs-offline-updates.h"
 #include "gs-folders.h"
@@ -51,7 +49,7 @@
 
 struct _GsApplication {
 	GtkApplication	 parent;
-	GsProfile	*profile;
+	AsProfile	*profile;
 	GCancellable	*cancellable;
 	GtkApplication	*application;
 	GtkCssProvider	*provider;
@@ -68,6 +66,12 @@ struct _GsApplication {
 
 G_DEFINE_TYPE (GsApplication, gs_application, GTK_TYPE_APPLICATION);
 
+GsPluginLoader *
+gs_application_get_plugin_loader (GsApplication *application)
+{
+	return application->plugin_loader;
+}
+
 gboolean
 gs_application_has_active_window (GsApplication *application)
 {
@@ -101,13 +105,14 @@ gs_application_init (GsApplication *application)
 		  _("Show profiling information for the service"), NULL },
 		{ "prefer-local", '\0', 0, G_OPTION_ARG_NONE, NULL,
 		  _("Prefer local file sources to AppStream"), NULL },
-		{ "version", 0, 0, G_OPTION_ARG_NONE, NULL, NULL, NULL },
+		{ "version", 0, 0, G_OPTION_ARG_NONE, NULL,
+		  _("Show version number"), NULL },
 		{ NULL }
 	};
 
 	g_application_add_main_option_entries (G_APPLICATION (application), options);
 
-	application->profile = gs_profile_new ();
+	application->profile = as_profile_new ();
 }
 
 static void
@@ -359,7 +364,7 @@ profile_activated (GSimpleAction *action,
 		   gpointer       data)
 {
 	GsApplication *app = GS_APPLICATION (data);
-	gs_profile_dump (app->profile);
+	as_profile_dump (app->profile);
 }
 
 static void
@@ -544,9 +549,9 @@ install_resources_activated (GSimpleAction *action,
 #endif
 	const gchar *mode;
 	const gchar *startup_id;
-	gchar **resources;
+	g_autofree gchar **resources = NULL;
 
-	g_variant_get (parameter, "(&s^as&s)", &mode, &resources, &startup_id);
+	g_variant_get (parameter, "(&s^a&s&s)", &mode, &resources, &startup_id);
 
 #ifdef GDK_WINDOWING_X11
 	display = gdk_display_get_default ();
@@ -584,8 +589,6 @@ gs_application_startup (GApplication *application)
 {
 	G_APPLICATION_CLASS (gs_application_parent_class)->startup (application);
 
-	g_type_ensure (GS_TYPE_BOX);
-
 	g_action_map_add_action_entries (G_ACTION_MAP (application),
 					 actions, G_N_ELEMENTS (actions),
 					 application);
@@ -689,6 +692,37 @@ gs_application_handle_local_options (GApplication *app, GVariantDict *options)
 	return -1;
 }
 
+static void
+gs_application_open (GApplication  *application,
+                     GFile        **files,
+                     gint           n_files,
+                     const gchar   *hint)
+{
+	GsApplication *app = GS_APPLICATION (application);
+	gint i;
+
+	for (i = 0; i < n_files; i++) {
+		g_autofree gchar *str = g_file_get_uri (files[i]);
+		g_autoptr(SoupURI) uri = NULL;
+
+		uri = soup_uri_new (str);
+		if (!SOUP_URI_IS_VALID (uri))
+			continue;
+
+		if (g_strcmp0 (soup_uri_get_scheme (uri), "appstream") == 0) {
+			const gchar *path = soup_uri_get_path (uri);
+
+			/* trim any leading slashes */
+			while (*path == '/')
+				path++;
+
+			g_action_group_activate_action (G_ACTION_GROUP (app),
+			                                "details",
+			                                g_variant_new ("(ss)", path, ""));
+		}
+	}
+}
+
 static void
 gs_application_class_init (GsApplicationClass *class)
 {
@@ -696,6 +730,7 @@ gs_application_class_init (GsApplicationClass *class)
 	G_APPLICATION_CLASS (class)->startup = gs_application_startup;
 	G_APPLICATION_CLASS (class)->activate = gs_application_activate;
 	G_APPLICATION_CLASS (class)->handle_local_options = gs_application_handle_local_options;
+	G_APPLICATION_CLASS (class)->open = gs_application_open;
 	G_APPLICATION_CLASS (class)->dbus_register = gs_application_dbus_register;
 	G_APPLICATION_CLASS (class)->dbus_unregister = gs_application_dbus_unregister;
 }
@@ -703,9 +738,9 @@ gs_application_class_init (GsApplicationClass *class)
 GsApplication *
 gs_application_new (void)
 {
-	g_set_prgname("org.gnome.Software");
 	return g_object_new (GS_APPLICATION_TYPE,
 			     "application-id", "org.gnome.Software",
+			     "flags", G_APPLICATION_HANDLES_OPEN,
 			     "inactivity-timeout", 12000,
 			     NULL);
 }
