@@ -24,12 +24,14 @@
 
 #include <gtk/gtk.h>
 
+#include "gs-plugin-loader.h"
+
 #define GS_APPLICATION_TYPE (gs_application_get_type ())
 
 G_DECLARE_FINAL_TYPE (GsApplication, gs_application, GS, APPLICATION, GtkApplication)
 
 GsApplication	*gs_application_new		(void);
-
+GsPluginLoader	*gs_application_get_plugin_loader	(GsApplication *application);
 gboolean	 gs_application_has_active_window	(GsApplication *application);
 
 #endif  /* __GS_APPLICATION_H */
