@@ -21,11 +21,11 @@
 
 #include "config.h"
 
+#include <appstream-glib.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <locale.h>
 
-#include "gs-profile.h"
 #include "gs-plugin-loader.h"
 #include "gs-plugin-loader-sync.h"
 
@@ -111,7 +111,7 @@ static GsPluginRefineFlags
 gs_cmd_refine_flag_from_string (const gchar *flag, GError **error)
 {
 	if (g_strcmp0 (flag, "all") == 0)
-		return 0xffff;
+		return G_MAXINT32;
 	if (g_strcmp0 (flag, "licence") == 0)
 		return GS_PLUGIN_REFINE_FLAGS_REQUIRE_LICENCE;
 	if (g_strcmp0 (flag, "url") == 0)
@@ -136,6 +136,10 @@ gs_cmd_refine_flag_from_string (const gchar *flag, GError **error)
 		return GS_PLUGIN_REFINE_FLAGS_REQUIRE_RELATED;
 	if (g_strcmp0 (flag, "menu-path") == 0)
 		return GS_PLUGIN_REFINE_FLAGS_REQUIRE_MENU_PATH;
+	if (g_strcmp0 (flag, "upgrade-removed") == 0)
+		return GS_PLUGIN_REFINE_FLAGS_REQUIRE_UPGRADE_REMOVED;
+	if (g_strcmp0 (flag, "provenance") == 0)
+		return GS_PLUGIN_REFINE_FLAGS_REQUIRE_PROVENANCE;
 	g_set_error (error,
 		     GS_PLUGIN_ERROR,
 		     GS_PLUGIN_ERROR_NOT_SUPPORTED,
@@ -185,7 +189,8 @@ main (int argc, char **argv)
 	g_autofree gchar *refine_flags_str = NULL;
 	g_autoptr(GsApp) app = NULL;
 	g_autoptr(GsPluginLoader) plugin_loader = NULL;
-	g_autoptr(GsProfile) profile = NULL;
+	g_autoptr(AsProfile) profile = NULL;
+	g_autoptr(AsProfileTask) ptask = NULL;
 	const GOptionEntry options[] = {
 		{ "show-results", '\0', 0, G_OPTION_ARG_NONE, &show_results,
 		  "Show the results for the action", NULL },
@@ -230,8 +235,8 @@ main (int argc, char **argv)
 		goto out;
 	}
 
-	profile = gs_profile_new ();
-	gs_profile_start (profile, "GsCmd");
+	profile = as_profile_new ();
+	ptask = as_profile_start_literal (profile, "GsCmd");
 
 	/* load plugins */
 	plugin_loader = gs_plugin_loader_new ();
@@ -271,6 +276,16 @@ main (int argc, char **argv)
 				break;
 			}
 		}
+	} else if (argc == 3 && g_strcmp0 (argv[1], "action-upgrade-download") == 0) {
+		app = gs_app_new (argv[2]);
+		gs_app_set_kind (app, GS_APP_KIND_DISTRO_UPGRADE);
+		ret = gs_plugin_loader_app_action (plugin_loader,
+						   app,
+						   GS_PLUGIN_LOADER_ACTION_UPGRADE_DOWNLOAD,
+						   NULL,
+						   &error);
+		if (ret)
+			gs_plugin_add_app (&list, app);
 	} else if (argc == 3 && g_strcmp0 (argv[1], "refine") == 0) {
 		app = gs_app_new (argv[2]);
 		for (i = 0; i < repeat; i++) {
@@ -283,6 +298,16 @@ main (int argc, char **argv)
 				break;
 		}
 		gs_plugin_add_app (&list, app);
+	} else if (argc == 3 && g_strcmp0 (argv[1], "launch") == 0) {
+		app = gs_app_new (argv[2]);
+		for (i = 0; i < repeat; i++) {
+			ret = gs_plugin_loader_app_launch (plugin_loader,
+							   app,
+							   NULL,
+							   &error);
+			if (!ret)
+				break;
+		}
 	} else if (argc == 3 && g_strcmp0 (argv[1], "filename-to-app") == 0) {
 		app = gs_plugin_loader_filename_to_app (plugin_loader,
 							argv[2],
@@ -307,6 +332,19 @@ main (int argc, char **argv)
 				break;
 			}
 		}
+	} else if (argc == 2 && g_strcmp0 (argv[1], "upgrades") == 0) {
+		for (i = 0; i < repeat; i++) {
+			if (list != NULL)
+				gs_plugin_list_free (list);
+			list = gs_plugin_loader_get_distro_upgrades (plugin_loader,
+								     refine_flags,
+								     NULL,
+								     &error);
+			if (list == NULL) {
+				ret = FALSE;
+				break;
+			}
+		}
 	} else if (argc == 2 && g_strcmp0 (argv[1], "sources") == 0) {
 		list = gs_plugin_loader_get_sources (plugin_loader,
 						     refine_flags,
@@ -389,7 +427,7 @@ main (int argc, char **argv)
 				     "Did not recognise option, use 'installed', "
 				     "'updates', 'popular', 'get-categories', "
 				     "'get-category-apps', 'filename-to-app', "
-				     "'sources', 'refresh' or 'search'");
+				     "'sources', 'refresh', 'launch' or 'search'");
 	}
 	if (!ret) {
 		g_print ("Failed: %s\n", error->message);
@@ -401,8 +439,8 @@ main (int argc, char **argv)
 		gs_cmd_show_results_categories (categories);
 	}
 out:
-	gs_profile_stop (profile, "GsCmd");
-	gs_profile_dump (profile);
+	if (profile != NULL)
+		as_profile_dump (profile);
 	g_option_context_free (context);
 	gs_plugin_list_free (list);
 	gs_plugin_list_free (categories);
