@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2012-2014 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2012-2015 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -192,6 +192,51 @@ gs_plugin_loader_get_updates (GsPluginLoader *plugin_loader,
 	return helper.list;
 }
 
+static void
+gs_plugin_loader_get_distro_upgrades_finish_sync (GsPluginLoader *plugin_loader,
+					  GAsyncResult *res,
+					  GsPluginLoaderHelper *helper)
+{
+	helper->list = gs_plugin_loader_get_distro_upgrades_finish (plugin_loader,
+								    res,
+								    helper->error);
+	g_main_loop_quit (helper->loop);
+}
+
+/**
+ * gs_plugin_loader_get_distro_upgrades:
+ **/
+GList *
+gs_plugin_loader_get_distro_upgrades (GsPluginLoader *plugin_loader,
+			      GsPluginRefineFlags flags,
+			      GCancellable *cancellable,
+			      GError **error)
+{
+	GsPluginLoaderHelper helper;
+
+	/* create temp object */
+	helper.context = g_main_context_new ();
+	helper.loop = g_main_loop_new (helper.context, FALSE);
+	helper.error = error;
+
+	g_main_context_push_thread_default (helper.context);
+
+	/* run async method */
+	gs_plugin_loader_get_distro_upgrades_async (plugin_loader,
+						    flags,
+						    cancellable,
+						    (GAsyncReadyCallback) gs_plugin_loader_get_distro_upgrades_finish_sync,
+						    &helper);
+	g_main_loop_run (helper.loop);
+
+	g_main_context_pop_thread_default (helper.context);
+
+	g_main_loop_unref (helper.loop);
+	g_main_context_unref (helper.context);
+
+	return helper.list;
+}
+
 static void
 gs_plugin_loader_get_sources_finish_sync (GsPluginLoader *plugin_loader,
 					  GAsyncResult *res,
@@ -254,8 +299,6 @@ gs_plugin_loader_get_popular_finish_sync (GsPluginLoader *plugin_loader,
 GList *
 gs_plugin_loader_get_popular (GsPluginLoader *plugin_loader,
 			      GsPluginRefineFlags flags,
-			      const gchar *category,
-			      const gchar *category_exclude,
 			      GCancellable *cancellable,
 			      GError **error)
 {
@@ -271,8 +314,6 @@ gs_plugin_loader_get_popular (GsPluginLoader *plugin_loader,
 	/* run async method */
 	gs_plugin_loader_get_popular_async (plugin_loader,
 					    flags,
-					    category,
-					    category_exclude,
 					    cancellable,
 					    (GAsyncReadyCallback) gs_plugin_loader_get_popular_finish_sync,
 					    &helper);
@@ -473,6 +514,54 @@ gs_plugin_loader_app_refine (GsPluginLoader *plugin_loader,
 	return helper.ret;
 }
 
+/**
+ * gs_plugin_loader_app_launch_finish_sync:
+ **/
+static void
+gs_plugin_loader_app_launch_finish_sync (GsPluginLoader *plugin_loader,
+					 GAsyncResult *res,
+					 GsPluginLoaderHelper *helper)
+{
+	helper->ret = gs_plugin_loader_app_launch_finish (plugin_loader,
+							  res,
+							  helper->error);
+	g_main_loop_quit (helper->loop);
+}
+
+/**
+ * gs_plugin_loader_app_launch:
+ **/
+gboolean
+gs_plugin_loader_app_launch (GsPluginLoader *plugin_loader,
+			     GsApp *app,
+			     GCancellable *cancellable,
+			     GError **error)
+{
+	GsPluginLoaderHelper helper;
+
+	/* create temp object */
+	helper.context = g_main_context_new ();
+	helper.loop = g_main_loop_new (helper.context, FALSE);
+	helper.error = error;
+
+	g_main_context_push_thread_default (helper.context);
+
+	/* run async method */
+	gs_plugin_loader_app_launch_async (plugin_loader,
+					   app,
+					   cancellable,
+					   (GAsyncReadyCallback) gs_plugin_loader_app_launch_finish_sync,
+					   &helper);
+	g_main_loop_run (helper.loop);
+
+	g_main_context_pop_thread_default (helper.context);
+
+	g_main_loop_unref (helper.loop);
+	g_main_context_unref (helper.context);
+
+	return helper.ret;
+}
+
 /**
  * gs_plugin_loader_app_action_finish_sync:
  **/
