@@ -22,6 +22,7 @@
 #ifndef __GS_PLUGIN_H
 #define __GS_PLUGIN_H
 
+#include <appstream-glib.h>
 #include <glib-object.h>
 #include <gmodule.h>
 #include <gio/gio.h>
@@ -29,7 +30,6 @@
 
 #include "gs-app.h"
 #include "gs-category.h"
-#include "gs-profile.h"
 
 G_BEGIN_DECLS
 
@@ -70,11 +70,12 @@ struct GsPlugin {
 	GsPluginPrivate		*priv;
 	guint			 pixbuf_size;
 	gint			 scale;
+	const gchar		*locale;
 	GsPluginStatusUpdate	 status_update_fn;
 	gpointer		 status_update_user_data;
 	GsPluginUpdatesChanged	 updates_changed_fn;
 	gpointer		 updates_changed_user_data;
-	GsProfile		*profile;
+	AsProfile		*profile;
 };
 
 typedef enum {
@@ -100,7 +101,9 @@ typedef enum {
 	GS_PLUGIN_REFINE_FLAGS_REQUIRE_MENU_PATH	= 1 << 12,
 	GS_PLUGIN_REFINE_FLAGS_REQUIRE_ADDONS		= 1 << 13,
 	GS_PLUGIN_REFINE_FLAGS_ALLOW_PACKAGES		= 1 << 14,
-	GS_PLUGIN_REFINE_FLAGS_ALLOW_NO_APPDATA		= 1 << 15,
+	GS_PLUGIN_REFINE_FLAGS_REQUIRE_UPDATE_SEVERITY	= 1 << 15,
+	GS_PLUGIN_REFINE_FLAGS_REQUIRE_UPGRADE_REMOVED	= 1 << 16,
+	GS_PLUGIN_REFINE_FLAGS_REQUIRE_PROVENANCE	= 1 << 17,
 	GS_PLUGIN_REFINE_FLAGS_LAST
 } GsPluginRefineFlags;
 
@@ -140,6 +143,10 @@ typedef gboolean	 (*GsPluginRefineFunc)		(GsPlugin	*plugin,
 							 GsPluginRefineFlags flags,
 							 GCancellable	*cancellable,
 							 GError		**error);
+typedef gboolean	 (*GsPluginLaunchFunc)		(GsPlugin	*plugin,
+							 GsApp		*app,
+							 GCancellable	*cancellable,
+							 GError		**error);
 typedef gboolean	 (*GsPluginRefreshFunc	)	(GsPlugin	*plugin,
 							 guint		 cache_age,
 							 GsPluginRefreshFlags flags,
@@ -178,6 +185,9 @@ void		 gs_plugin_status_update		(GsPlugin	*plugin,
 void		 gs_plugin_progress_update		(GsPlugin	*plugin,
 							 GsApp		*app,
 							 guint		 percentage);
+gboolean	 gs_plugin_app_launch			(GsPlugin	*plugin,
+							 GsApp		*app,
+							 GError		**error);
 void		 gs_plugin_updates_changed		(GsPlugin	*plugin);
 const gchar	*gs_plugin_status_to_string		(GsPluginStatus	 status);
 gboolean	 gs_plugin_add_search			(GsPlugin	*plugin,
@@ -204,6 +214,10 @@ gboolean	 gs_plugin_add_updates			(GsPlugin	*plugin,
 							 GList		**list,
 							 GCancellable	*cancellable,
 							 GError		**error);
+gboolean	 gs_plugin_add_distro_upgrades		(GsPlugin	*plugin,
+							 GList		**list,
+							 GCancellable	*cancellable,
+							 GError		**error);
 gboolean	 gs_plugin_add_sources			(GsPlugin	*plugin,
 							 GList		**list,
 							 GCancellable	*cancellable,
@@ -234,6 +248,10 @@ gboolean	 gs_plugin_refine			(GsPlugin	*plugin,
 							 GsPluginRefineFlags flags,
 							 GCancellable	*cancellable,
 							 GError		**error);
+gboolean	 gs_plugin_launch			(GsPlugin	*plugin,
+							 GsApp		*app,
+							 GCancellable	*cancellable,
+							 GError		**error);
 gboolean	 gs_plugin_app_install			(GsPlugin	*plugin,
 							 GsApp		*app,
 							 GCancellable	*cancellable,
@@ -246,6 +264,14 @@ gboolean	 gs_plugin_app_set_rating		(GsPlugin	*plugin,
 							 GsApp		*app,
 							 GCancellable	*cancellable,
 							 GError		**error);
+gboolean	 gs_plugin_app_update			(GsPlugin	*plugin,
+							 GsApp		*app,
+							 GCancellable	*cancellable,
+							 GError		**error);
+gboolean	 gs_plugin_app_upgrade_download		(GsPlugin	*plugin,
+							 GsApp		*app,
+							 GCancellable	*cancellable,
+							 GError		**error);
 gboolean	 gs_plugin_refresh			(GsPlugin	*plugin,
 							 guint		 cache_age,
 							 GsPluginRefreshFlags flags,
