@@ -24,7 +24,6 @@
 
 #include <string.h>
 #include <glib/gi18n.h>
-#include <gio/gdesktopappinfo.h>
 #include <appstream-glib.h>
 
 #include "gs-utils.h"
@@ -38,7 +37,8 @@
 
 typedef enum {
 	GS_SHELL_DETAILS_STATE_LOADING,
-	GS_SHELL_DETAILS_STATE_READY
+	GS_SHELL_DETAILS_STATE_READY,
+	GS_SHELL_DETAILS_STATE_FAILED
 } GsShellDetailsState;
 
 struct _GsShellDetails
@@ -83,12 +83,21 @@ struct _GsShellDetails
 	GtkWidget		*label_details_size_value;
 	GtkWidget		*label_details_updated_value;
 	GtkWidget		*label_details_version_value;
+	GtkWidget		*label_failed;
 	GtkWidget		*label_pending;
 	GtkWidget		*list_box_addons;
 	GtkWidget		*scrolledwindow_details;
 	GtkWidget		*spinner_details;
 	GtkWidget		*spinner_install_remove;
 	GtkWidget		*stack_details;
+	GtkWidget		*image_details_kudo_docs;
+	GtkWidget		*image_details_kudo_integration;
+	GtkWidget		*image_details_kudo_translated;
+	GtkWidget		*image_details_kudo_updated;
+	GtkWidget		*label_details_kudo_docs;
+	GtkWidget		*label_details_kudo_integration;
+	GtkWidget		*label_details_kudo_translated;
+	GtkWidget		*label_details_kudo_updated;
 };
 
 G_DEFINE_TYPE (GsShellDetails, gs_shell_details, GS_TYPE_PAGE)
@@ -107,6 +116,7 @@ gs_shell_details_set_state (GsShellDetails *self,
 		gtk_widget_show (self->spinner_details);
 		break;
 	case GS_SHELL_DETAILS_STATE_READY:
+	case GS_SHELL_DETAILS_STATE_FAILED:
 		gs_stop_spinner (GTK_SPINNER (self->spinner_details));
 		gtk_widget_hide (self->spinner_details);
 		break;
@@ -122,6 +132,9 @@ gs_shell_details_set_state (GsShellDetails *self,
 	case GS_SHELL_DETAILS_STATE_READY:
 		gtk_stack_set_visible_child_name (GTK_STACK (self->stack_details), "ready");
 		break;
+	case GS_SHELL_DETAILS_STATE_FAILED:
+		gtk_stack_set_visible_child_name (GTK_STACK (self->stack_details), "failed");
+		break;
 	default:
 		g_assert_not_reached ();
 	}
@@ -186,6 +199,7 @@ gs_shell_details_switch_to (GsShellDetails *self)
 	case AS_APP_STATE_INSTALLED:
 	case AS_APP_STATE_REMOVING:
 	case AS_APP_STATE_UPDATABLE:
+	case AS_APP_STATE_UPDATABLE_LIVE:
 		gtk_widget_set_visible (self->button_install, FALSE);
 		break;
 	case AS_APP_STATE_UNAVAILABLE:
@@ -206,6 +220,27 @@ gs_shell_details_switch_to (GsShellDetails *self)
 		g_assert_not_reached ();
 	}
 
+	/* launch button */
+	switch (gs_app_get_state (self->app)) {
+	case AS_APP_STATE_INSTALLED:
+	case AS_APP_STATE_UPDATABLE:
+	case AS_APP_STATE_UPDATABLE_LIVE:
+		if (gs_app_get_id_kind (self->app) == AS_ID_KIND_DESKTOP ||
+		    gs_app_get_id_kind (self->app) == AS_ID_KIND_WEB_APP) {
+			gtk_widget_set_visible (self->button_details_launch, TRUE);
+		} else {
+			gtk_widget_set_visible (self->button_details_launch, FALSE);
+		}
+		break;
+	default:
+		gtk_widget_set_visible (self->button_details_launch, FALSE);
+		break;
+	}
+
+	/* don't show the launch button if the app doesn't have a desktop ID */
+	if (gs_app_get_id (self->app) == NULL)
+		gtk_widget_set_visible (self->button_details_launch, FALSE);
+
 	/* remove button */
 	if (kind == GS_APP_KIND_SYSTEM) {
 		gtk_widget_set_visible (self->button_remove, FALSE);
@@ -213,9 +248,14 @@ gs_shell_details_switch_to (GsShellDetails *self)
 		switch (state) {
 		case AS_APP_STATE_INSTALLED:
 		case AS_APP_STATE_UPDATABLE:
+		case AS_APP_STATE_UPDATABLE_LIVE:
 			gtk_widget_set_visible (self->button_remove, TRUE);
 			gtk_widget_set_sensitive (self->button_remove, TRUE);
-			gtk_style_context_add_class (gtk_widget_get_style_context (self->button_remove), "destructive-action");
+			/* Mark the button as destructive only if Launch is not visible */
+			if (gtk_widget_get_visible (self->button_details_launch))
+				gtk_style_context_remove_class (gtk_widget_get_style_context (self->button_remove), "destructive-action");
+			else
+				gtk_style_context_add_class (gtk_widget_get_style_context (self->button_remove), "destructive-action");
 			/* TRANSLATORS: button text in the header when an application can be erased */
 			gtk_button_set_label (GTK_BUTTON (self->button_remove), _("_Remove"));
 			break;
@@ -267,6 +307,7 @@ gs_shell_details_switch_to (GsShellDetails *self)
 		case AS_APP_STATE_AVAILABLE:
 		case AS_APP_STATE_QUEUED_FOR_INSTALL:
 		case AS_APP_STATE_UPDATABLE:
+		case AS_APP_STATE_UPDATABLE_LIVE:
 		case AS_APP_STATE_UNAVAILABLE:
 		case AS_APP_STATE_AVAILABLE_LOCAL:
 		case AS_APP_STATE_INSTALLING:
@@ -380,6 +421,8 @@ gs_shell_details_refresh_screenshots (GsShellDetails *self)
 			label = gtk_label_new (as_screenshot_get_caption (ss, NULL));
 			g_object_set (label,
 				      "xalign", 0.0,
+				      "max-width-chars", 10,
+				      "wrap", TRUE,
 				      NULL);
 			gtk_box_pack_start (GTK_BOX (self->box_details_screenshot_main), label, FALSE, FALSE, 0);
 			gtk_widget_set_visible (label, TRUE);
@@ -496,7 +539,7 @@ gs_shell_details_set_description (GsShellDetails *self, const gchar *tmp)
 	for (i = 0; split[i] != NULL; i++) {
 		para = gtk_label_new (split[i]);
 		gtk_label_set_line_wrap (GTK_LABEL (para), TRUE);
-		gtk_label_set_max_width_chars (GTK_LABEL (para), 80);
+		gtk_label_set_max_width_chars (GTK_LABEL (para), 40);
 		gtk_label_set_selectable (GTK_LABEL (para), TRUE);
 		gtk_widget_set_visible (para, TRUE);
 		gtk_widget_set_can_focus (para, FALSE);
@@ -513,6 +556,18 @@ gs_shell_details_set_description (GsShellDetails *self, const gchar *tmp)
 	}
 }
 
+static void
+gs_shell_details_set_sensitive (GtkWidget *widget, gboolean is_active)
+{
+	GtkStyleContext *style_context;
+	style_context = gtk_widget_get_style_context (widget);
+	if (!is_active) {
+		gtk_style_context_add_class (style_context, "dim-label");
+	} else {
+		gtk_style_context_remove_class (style_context, "dim-label");
+	}
+}
+
 /**
  * gs_shell_details_refresh_all:
  **/
@@ -525,7 +580,9 @@ gs_shell_details_refresh_all (GsShellDetails *self)
 	GtkWidget *widget;
 	const gchar *tmp;
 	gchar **menu_path;
+	guint64 kudos;
 	guint64 updated;
+	guint64 user_integration_bf;
 	g_autoptr(GError) error = NULL;
 
 	/* change widgets */
@@ -552,21 +609,7 @@ gs_shell_details_refresh_all (GsShellDetails *self)
 	gs_shell_details_set_description (self, tmp);
 
 	/* set the icon */
-	tmp = gs_app_get_metadata_item (self->app, "DataDir::desktop-icon");
-	if (tmp != NULL) {
-		pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
-						   tmp, 96,
-						   GTK_ICON_LOOKUP_USE_BUILTIN |
-						   GTK_ICON_LOOKUP_FORCE_SIZE,
-						   &error);
-		if (pixbuf == NULL) {
-			g_warning ("Failed to load desktop icon: %s",
-				   error->message);
-			g_clear_error (&error);
-		}
-	}
-	if (pixbuf == NULL)
-		pixbuf = gs_app_get_pixbuf (self->app);
+	pixbuf = gs_app_get_pixbuf (self->app);
 	if (pixbuf != NULL) {
 		gs_image_set_from_pixbuf (GTK_IMAGE (self->application_details_icon), pixbuf);
 		gtk_widget_set_visible (self->application_details_icon, TRUE);
@@ -689,6 +732,35 @@ gs_shell_details_refresh_all (GsShellDetails *self)
 		break;
 	}
 
+	/* set MyLanguage kudo */
+	kudos = gs_app_get_kudos (self->app);
+	gs_shell_details_set_sensitive (self->image_details_kudo_translated,
+					kudos & GS_APP_KUDO_MY_LANGUAGE);
+	gs_shell_details_set_sensitive (self->label_details_kudo_translated,
+					kudos & GS_APP_KUDO_MY_LANGUAGE);
+
+	/* set RecentRelease kudo */
+	gs_shell_details_set_sensitive (self->image_details_kudo_updated,
+					kudos & GS_APP_KUDO_RECENT_RELEASE);
+	gs_shell_details_set_sensitive (self->label_details_kudo_updated,
+					kudos & GS_APP_KUDO_RECENT_RELEASE);
+
+	/* set UserDocs kudo */
+	gs_shell_details_set_sensitive (self->image_details_kudo_docs,
+					kudos & GS_APP_KUDO_INSTALLS_USER_DOCS);
+	gs_shell_details_set_sensitive (self->label_details_kudo_docs,
+					kudos & GS_APP_KUDO_INSTALLS_USER_DOCS);
+
+	/* any of the various integration kudos */
+	user_integration_bf = GS_APP_KUDO_SEARCH_PROVIDER |
+			      GS_APP_KUDO_USES_NOTIFICATIONS |
+			      GS_APP_KUDO_USES_APP_MENU |
+			      GS_APP_KUDO_HIGH_CONTRAST;
+	gs_shell_details_set_sensitive (self->image_details_kudo_integration,
+					kudos & user_integration_bf);
+	gs_shell_details_set_sensitive (self->label_details_kudo_integration,
+					kudos & user_integration_bf);
+
 	/* don't show a missing rating on a local file */
 	if (gs_app_get_state (self->app) == AS_APP_STATE_AVAILABLE_LOCAL &&
 	    gs_app_get_rating (self->app) < 0)
@@ -698,27 +770,6 @@ gs_shell_details_refresh_all (GsShellDetails *self)
 	gtk_widget_set_sensitive (self->star,
 				  gs_app_get_state (self->app) == AS_APP_STATE_INSTALLED);
 
-	/* only show launch button when the application is installed */
-	switch (gs_app_get_state (self->app)) {
-	case AS_APP_STATE_INSTALLED:
-	case AS_APP_STATE_UPDATABLE:
-		if (gs_app_get_id_kind (self->app) == AS_ID_KIND_DESKTOP ||
-		    gs_app_get_id_kind (self->app) == AS_ID_KIND_WEB_APP) {
-			gtk_widget_set_visible (self->button_details_launch, TRUE);
-		} else {
-			gtk_widget_set_visible (self->button_details_launch, FALSE);
-		}
-		break;
-	default:
-		gtk_widget_set_visible (self->button_details_launch, FALSE);
-		break;
-	}
-
-	/* don't show the launch button if the app doesn't have a desktop ID */
-	tmp = gs_app_get_id (self->app);
-	if (tmp == NULL)
-		gtk_widget_set_visible (self->button_details_launch, FALSE);
-
 	/* make history button insensitive if there is none */
 	history = gs_app_get_history (self->app);
 	switch (gs_app_get_id_kind (self->app)) {
@@ -798,6 +849,7 @@ gs_shell_details_refresh_all (GsShellDetails *self)
 	switch (gs_app_get_state (self->app)) {
 	case AS_APP_STATE_INSTALLED:
 	case AS_APP_STATE_UPDATABLE:
+	case AS_APP_STATE_UPDATABLE_LIVE:
 		gtk_widget_set_visible (self->label_addons_uninstalled_app, FALSE);
 		break;
 	default:
@@ -884,6 +936,15 @@ gs_shell_details_app_refine_cb (GObject *source,
 		g_warning ("failed to refine %s: %s",
 			   gs_app_get_id (self->app),
 			   error->message);
+	}
+
+	if (gs_app_get_kind (self->app) == GS_APP_KIND_UNKNOWN ||
+	    gs_app_get_state (self->app) == AS_APP_STATE_UNKNOWN) {
+		g_autofree gchar *str = NULL;
+
+		str = g_strdup_printf (_("Could not find '%s'"), gs_app_get_id (self->app));
+		gtk_label_set_text (GTK_LABEL (self->label_failed), str);
+		gs_shell_details_set_state (self, GS_SHELL_DETAILS_STATE_FAILED);
 		return;
 	}
 
@@ -1109,6 +1170,7 @@ gs_shell_details_addon_selected_cb (GsAppAddonRow *row,
 	switch (gs_app_get_state (self->app)) {
 	case AS_APP_STATE_INSTALLED:
 	case AS_APP_STATE_UPDATABLE:
+	case AS_APP_STATE_UPDATABLE_LIVE:
 		if (gs_app_addon_row_get_selected (row)) {
 			gs_page_install_app (GS_PAGE (self), addon);
 		} else {
@@ -1124,32 +1186,33 @@ gs_shell_details_addon_selected_cb (GsAppAddonRow *row,
 	}
 }
 
+/**
+ * gs_shell_details_filename_to_app_cb:
+ **/
+static void
+gs_shell_details_app_launch_cb (GObject *source,
+				GAsyncResult *res,
+				gpointer user_data)
+{
+	GsShellDetails *self = GS_SHELL_DETAILS (user_data);
+	g_autoptr(GError) error = NULL;
+	if (!gs_plugin_loader_app_launch_finish (self->plugin_loader, res, &error)) {
+		g_warning ("failed to launch GsApp: %s", error->message);
+		return;
+	}
+}
+
 /**
  * gs_shell_details_app_launch_button_cb:
  **/
 static void
 gs_shell_details_app_launch_button_cb (GtkWidget *widget, GsShellDetails *self)
 {
-	GdkDisplay *display;
-	const gchar *desktop_id;
-	g_autoptr(GError) error = NULL;
-	g_autoptr(GAppInfo) appinfo = NULL;
-	g_autoptr(GAppLaunchContext) context = NULL;
-
-	desktop_id = gs_app_get_id (self->app);
-	if (desktop_id == NULL) {
-		g_warning ("no such desktop file: %s", desktop_id);
-		return;
-	}
-	appinfo = G_APP_INFO (g_desktop_app_info_new (desktop_id));
-	if (appinfo == NULL) {
-		g_warning ("no such desktop file: %s", desktop_id);
-		return;
-	}
-	display = gdk_display_get_default ();
-	context = G_APP_LAUNCH_CONTEXT (gdk_display_get_app_launch_context (display));
-	if (!g_app_info_launch (appinfo, NULL, context, &error))
-		g_warning ("launching %s failed: %s", desktop_id, error->message);
+	gs_plugin_loader_app_launch_async (self->plugin_loader,
+					   self->app,
+					   self->cancellable,
+					   gs_shell_details_app_launch_cb,
+					   self);
 }
 
 /**
@@ -1343,12 +1406,21 @@ gs_shell_details_class_init (GsShellDetailsClass *klass)
 	gtk_widget_class_bind_template_child (widget_class, GsShellDetails, label_details_size_value);
 	gtk_widget_class_bind_template_child (widget_class, GsShellDetails, label_details_updated_value);
 	gtk_widget_class_bind_template_child (widget_class, GsShellDetails, label_details_version_value);
+	gtk_widget_class_bind_template_child (widget_class, GsShellDetails, label_failed);
 	gtk_widget_class_bind_template_child (widget_class, GsShellDetails, label_pending);
 	gtk_widget_class_bind_template_child (widget_class, GsShellDetails, list_box_addons);
 	gtk_widget_class_bind_template_child (widget_class, GsShellDetails, scrolledwindow_details);
 	gtk_widget_class_bind_template_child (widget_class, GsShellDetails, spinner_details);
 	gtk_widget_class_bind_template_child (widget_class, GsShellDetails, spinner_install_remove);
 	gtk_widget_class_bind_template_child (widget_class, GsShellDetails, stack_details);
+	gtk_widget_class_bind_template_child (widget_class, GsShellDetails, image_details_kudo_docs);
+	gtk_widget_class_bind_template_child (widget_class, GsShellDetails, image_details_kudo_integration);
+	gtk_widget_class_bind_template_child (widget_class, GsShellDetails, image_details_kudo_translated);
+	gtk_widget_class_bind_template_child (widget_class, GsShellDetails, image_details_kudo_updated);
+	gtk_widget_class_bind_template_child (widget_class, GsShellDetails, label_details_kudo_docs);
+	gtk_widget_class_bind_template_child (widget_class, GsShellDetails, label_details_kudo_integration);
+	gtk_widget_class_bind_template_child (widget_class, GsShellDetails, label_details_kudo_translated);
+	gtk_widget_class_bind_template_child (widget_class, GsShellDetails, label_details_kudo_updated);
 }
 
 /**
@@ -1360,8 +1432,7 @@ gs_shell_details_init (GsShellDetails *self)
 	gtk_widget_init_template (GTK_WIDGET (self));
 
 	/* setup networking */
-	self->session = soup_session_new_with_options (SOUP_SESSION_USER_AGENT,
-	                                               "gnome-software",
+	self->session = soup_session_new_with_options (SOUP_SESSION_USER_AGENT, gs_user_agent (),
 	                                               NULL);
 
 	gtk_list_box_set_header_func (GTK_LIST_BOX (self->list_box_addons),
