@@ -55,7 +55,6 @@ struct _GsShellExtras
 
 	GsPluginLoader		 *plugin_loader;
 	GtkBuilder		 *builder;
-	GCancellable		 *cancellable;
 	GCancellable		 *search_cancellable;
 	GsShell			 *shell;
 	GsShellExtrasState	  state;
@@ -697,8 +696,7 @@ gs_shell_extras_load (GsShellExtras *self, GPtrArray *array_search_data)
 			                                     search_data->search_filename,
 			                                     GS_PLUGIN_REFINE_FLAGS_DEFAULT |
 			                                     GS_PLUGIN_REFINE_FLAGS_REQUIRE_RATING |
-			                                     GS_PLUGIN_REFINE_FLAGS_ALLOW_PACKAGES |
-			                                     GS_PLUGIN_REFINE_FLAGS_ALLOW_NO_APPDATA,
+			                                     GS_PLUGIN_REFINE_FLAGS_ALLOW_PACKAGES,
 			                                     self->search_cancellable,
 			                                     search_files_cb,
 			                                     search_data);
@@ -708,8 +706,7 @@ gs_shell_extras_load (GsShellExtras *self, GPtrArray *array_search_data)
 			                                        search_data->package_filename,
 			                                        GS_PLUGIN_REFINE_FLAGS_DEFAULT |
 			                                        GS_PLUGIN_REFINE_FLAGS_REQUIRE_RATING |
-			                                        GS_PLUGIN_REFINE_FLAGS_ALLOW_PACKAGES |
-			                                        GS_PLUGIN_REFINE_FLAGS_ALLOW_NO_APPDATA,
+			                                        GS_PLUGIN_REFINE_FLAGS_ALLOW_PACKAGES,
 			                                        self->search_cancellable,
 			                                        filename_to_app_cb,
 			                                        search_data);
@@ -723,8 +720,7 @@ gs_shell_extras_load (GsShellExtras *self, GPtrArray *array_search_data)
 			                                             GS_PLUGIN_REFINE_FLAGS_REQUIRE_SETUP_ACTION |
 			                                             GS_PLUGIN_REFINE_FLAGS_REQUIRE_DESCRIPTION |
 			                                             GS_PLUGIN_REFINE_FLAGS_REQUIRE_RATING |
-			                                             GS_PLUGIN_REFINE_FLAGS_ALLOW_PACKAGES |
-			                                             GS_PLUGIN_REFINE_FLAGS_ALLOW_NO_APPDATA,
+			                                             GS_PLUGIN_REFINE_FLAGS_ALLOW_PACKAGES,
 			                                             self->search_cancellable,
 			                                             get_search_what_provides_cb,
 			                                             search_data);
@@ -1128,7 +1124,6 @@ gs_shell_extras_setup (GsShellExtras *self,
 
 	self->plugin_loader = g_object_ref (plugin_loader);
 	self->builder = g_object_ref (builder);
-	self->cancellable = g_object_ref (cancellable);
 
 	g_signal_connect (self->list_box_results, "row-activated",
 			  G_CALLBACK (row_activated_cb), self);
@@ -1162,7 +1157,6 @@ gs_shell_extras_dispose (GObject *object)
 	g_clear_object (&self->vendor);
 	g_clear_object (&self->builder);
 	g_clear_object (&self->plugin_loader);
-	g_clear_object (&self->cancellable);
 
 	g_clear_pointer (&self->array_search_data, g_ptr_array_unref);
 
