@@ -260,9 +260,6 @@ gs_shell_updates_update_ui_state (GsShellUpdates *self)
 		gtk_label_set_label (GTK_LABEL (self->label_updates_spinner), spinner_str);
 		break;
 	case GS_SHELL_UPDATES_STATE_ACTION_GET_UPDATES:
-		/* TRANSLATORS: this is when the updates panel is starting up */
-		gtk_label_set_label (GTK_LABEL (self->label_updates_spinner), _("Checking for updates…"));
-		break;
 	case GS_SHELL_UPDATES_STATE_ACTION_REFRESH_HAS_UPDATES:
 	case GS_SHELL_UPDATES_STATE_NO_UPDATES:
 	case GS_SHELL_UPDATES_STATE_MANAGED:
@@ -509,6 +506,7 @@ gs_shell_updates_load (GsShellUpdates *self)
 	gs_container_remove_all (GTK_CONTAINER (self->list_box_updates));
 	refine_flags = GS_PLUGIN_REFINE_FLAGS_DEFAULT |
 		       GS_PLUGIN_REFINE_FLAGS_REQUIRE_UPDATE_DETAILS |
+		       GS_PLUGIN_REFINE_FLAGS_REQUIRE_PROVENANCE |
 		       GS_PLUGIN_REFINE_FLAGS_REQUIRE_VERSION;
 	gs_shell_updates_set_state (self,
 				    GS_SHELL_UPDATES_STATE_ACTION_GET_UPDATES);
@@ -596,6 +594,18 @@ gs_shell_updates_activated_cb (GtkListBox *list_box,
 	show_update_details (app, self);
 }
 
+/**
+ * gs_shell_updates_button_clicked_cb:
+ **/
+static void
+gs_shell_updates_button_clicked_cb (GsUpdateList *update_list,
+				    GsApp *app,
+				    GsShellUpdates *self)
+{
+	if (gs_app_get_state (app) == AS_APP_STATE_UPDATABLE_LIVE)
+		gs_page_update_app (GS_PAGE (self), app);
+}
+
 /**
  * gs_shell_updates_refresh_cb:
  **/
@@ -949,6 +959,8 @@ gs_shell_updates_setup (GsShellUpdates *self,
 	/* setup updates */
 	g_signal_connect (self->list_box_updates, "row-activated",
 			  G_CALLBACK (gs_shell_updates_activated_cb), self);
+	g_signal_connect (self->list_box_updates, "button-clicked",
+			  G_CALLBACK (gs_shell_updates_button_clicked_cb), self);
 
 	widget = GTK_WIDGET (gtk_builder_get_object (self->builder, "button_update_all"));
 	g_signal_connect (widget, "clicked", G_CALLBACK (gs_shell_updates_button_update_all_cb), self);
