@@ -232,7 +232,10 @@ gs_update_dialog_show_installed_updates (GsUpdateDialog *dialog)
 		date = g_date_time_new_from_unix_utc (time_updates_installed);
 		date_str = g_date_time_format (date, "%x");
 
-		/* TRANSLATORS: this is the subtitle of the installed updates dialog window */
+		/* TRANSLATORS: this is the subtitle of the installed updates dialog window.
+		   %s will be replaced by the date when the updates were installed.
+		   The date format is defined by the locale's preferred date representation
+		   ("%x" in strftime.) */
 		subtitle = g_strdup_printf (_("Installed on %s"), date_str);
 		header = gtk_dialog_get_header_bar (GTK_DIALOG (dialog));
 		gtk_header_bar_set_subtitle (GTK_HEADER_BAR (header), subtitle);
@@ -304,21 +307,23 @@ gs_update_dialog_show_update_details (GsUpdateDialog *dialog, GsApp *app)
 			label = gtk_label_new (gs_app_get_source_default (app_related));
 			g_object_set (label,
 				      "margin-start", 20,
-				      "margin-end", 20,
+				      "margin-end", 0,
 				      "margin-top", 6,
 				      "margin-bottom", 6,
 				      "xalign", 0.0,
+				      "ellipsize", PANGO_ELLIPSIZE_END,
 				      NULL);
 			gtk_widget_set_halign (label, GTK_ALIGN_START);
 			gtk_widget_set_valign (label, GTK_ALIGN_CENTER);
 			gtk_box_pack_start (GTK_BOX (row), label, TRUE, TRUE, 0);
 			label = gtk_label_new (gs_app_get_update_version (app_related));
 			g_object_set (label,
-				      "margin-start", 20,
+				      "margin-start", 0,
 				      "margin-end", 20,
 				      "margin-top", 6,
 				      "margin-bottom", 6,
 				      "xalign", 1.0,
+				      "ellipsize", PANGO_ELLIPSIZE_END,
 				      NULL);
 			gtk_widget_set_halign (label, GTK_ALIGN_END);
 			gtk_widget_set_valign (label, GTK_ALIGN_CENTER);
