@@ -34,6 +34,8 @@ void	 gs_stop_spinner		(GtkSpinner	*spinner);
 void	 gs_container_remove_all	(GtkContainer	*container);
 void	 gs_grab_focus_when_mapped	(GtkWidget	*widget);
 
+guint	 gs_utils_get_file_age		(const gchar	*fn);
+
 void	 gs_app_notify_installed	(GsApp		*app);
 void	 gs_app_notify_failed_modal	(GsApp		*app,
 					 GtkWindow	*parent_window,
@@ -57,6 +59,7 @@ void	gs_image_set_from_pixbuf_with_scale	(GtkImage		*image,
 						 gint			 scale);
 void	gs_image_set_from_pixbuf		(GtkImage		*image,
 						 const GdkPixbuf	*pixbuf);
+const gchar	 *gs_user_agent			(void);
 
 G_END_DECLS
 
