@@ -30,6 +30,13 @@
 #include <gs-plugin.h>
 #include <gs-utils.h>
 
+/*
+ * SECTION:
+ * Uses epiphany to launch web applications.
+ *
+ * If the epiphany binary is not present then it self-disables.
+ */
+
 /**
  * gs_plugin_get_name:
  */
@@ -183,9 +190,10 @@ gs_plugin_app_install (GsPlugin *plugin, GsApp *app,
 		return FALSE;
 
 	/* symlink it to somewhere the shell will notice */
-	app_desktop = g_strdup_printf ("%s/applications/%s",
-				       g_get_user_data_dir (),
-				       gs_app_get_id (app));
+	app_desktop = g_build_filename (g_get_user_data_dir (),
+	                                "applications",
+	                                gs_app_get_id (app),
+	                                NULL);
 	symlink_desktop = g_file_new_for_path (app_desktop);
 	ret = g_file_make_symbolic_link (symlink_desktop,
 					 epi_desktop,
@@ -228,9 +236,10 @@ gs_plugin_app_remove (GsPlugin *plugin, GsApp *app,
 
 	/* remove the shared desktop file */
 	basename = g_file_get_basename (file_epi);
-	app_desktop = g_strdup_printf ("%s/applications/%s",
-				   g_get_user_data_dir (),
-				   gs_app_get_id (app));
+	app_desktop = g_build_filename (g_get_user_data_dir (),
+	                                "applications",
+	                                gs_app_get_id (app),
+	                                NULL);
 	file_app = g_file_new_for_path (app_desktop);
 	if (!g_file_delete (file_app, NULL, error))
 		return FALSE;
@@ -293,3 +302,18 @@ gs_plugin_refine (GsPlugin *plugin,
 	}
 	return TRUE;
 }
+
+/**
+ * gs_plugin_launch:
+ */
+gboolean
+gs_plugin_launch (GsPlugin *plugin,
+		  GsApp *app,
+		  GCancellable *cancellable,
+		  GError **error)
+{
+	/* only process this app if was created by this plugin */
+	if (g_strcmp0 (gs_app_get_management_plugin (app), "Epiphany") != 0)
+		return TRUE;
+	return gs_plugin_app_launch (plugin, app, error);
+}
