@@ -21,6 +21,7 @@
 
 #include <config.h>
 
+#include <json-glib/json-glib.h>
 #include <libsoup/soup.h>
 #include <string.h>
 #include <sqlite3.h>
@@ -29,6 +30,13 @@
 #include <gs-plugin.h>
 #include <gs-utils.h>
 
+/*
+ * SECTION:
+ * Gets and sets the application rating using the fedora-tagger service.
+ *
+ * It will self-disable if not run on a Fedora system.
+ */
+
 struct GsPluginPrivate {
 	SoupSession		*session;
 	gchar			*db_path;
@@ -57,12 +65,12 @@ void
 gs_plugin_initialize (GsPlugin *plugin)
 {
 	plugin->priv = GS_PLUGIN_GET_PRIVATE (GsPluginPrivate);
-	plugin->priv->db_path = g_build_filename (g_get_home_dir (),
-						  ".local",
-						  "share",
+	plugin->priv->db_path = g_build_filename (g_get_user_data_dir (),
 						  "gnome-software",
 						  "fedora-tagger.db",
 						  NULL);
+	plugin->priv->session = soup_session_new_with_options (SOUP_SESSION_USER_AGENT, gs_user_agent (),
+	                                                       NULL);
 
 	/* check that we are running on Fedora */
 	if (!gs_plugin_check_distro_id (plugin, "fedora")) {
@@ -100,79 +108,36 @@ gs_plugin_destroy (GsPlugin *plugin)
 
 /**
  * gs_plugin_parse_json:
- *
- * This is a quick and dirty JSON parser that extracts one line from the
- * JSON formatted data. Sorry JsonGlib, you look awesome, but you're just too
- * heavy for an error message.
  */
 static gchar *
-gs_plugin_parse_json (const gchar *data, gsize data_len, const gchar *key)
+gs_plugin_parse_json (const gchar *data, gsize data_len, const gchar *key, GError **error)
 {
-	gchar *key_full;
+	JsonNode *root;
+	JsonObject *root_object;
+	JsonParser *parser;
+	gboolean ret;
 	gchar *value = NULL;
-	guint i;
-	gchar *tmp;
-	guint len;
-	g_autoptr(GString) string = NULL;
-	g_auto(GStrv) split = NULL;
 
-	/* format the key to match what JSON returns */
-	key_full = g_strdup_printf ("\"%s\":", key);
+	parser = json_parser_new ();
 
-	/* replace escaping with something sane */
-	string = g_string_new_len (data, data_len);
-	gs_string_replace (string, "\\\"", "'");
+	ret = json_parser_load_from_data (parser, data, data_len, error);
+	if (!ret)
+		goto out;
 
-	/* find the line that corresponds to our key */
-	split = g_strsplit (string->str, "\n", -1);
-	for (i = 0; split[i] != NULL; i++) {
-		tmp = g_strchug (split[i]);
-		if (g_str_has_prefix (tmp, key_full)) {
-			tmp += strlen (key_full);
+	root = json_parser_get_root (parser);
+	if (root == NULL)
+		goto out;
 
-			/* remove leading chars */
-			tmp = g_strstrip (tmp);
-			if (tmp[0] == '\"')
-				tmp += 1;
+	root_object = json_node_get_object (root);
+	if (root_object == NULL)
+		goto out;
 
-			/* remove trailing chars */
-			len = strlen (tmp);
-			if (tmp[len-1] == ',')
-				len -= 1;
-			if (tmp[len-1] == '\"')
-				len -= 1;
-			value = g_strndup (tmp, len);
-		}
-	}
+	value = g_strdup (json_object_get_string_member (root_object, key));
+out:
+	g_object_unref (parser);
 	return value;
 }
 
-
-/**
- * gs_plugin_setup_networking:
- */
-static gboolean
-gs_plugin_setup_networking (GsPlugin *plugin, GError **error)
-{
-	/* already set up */
-	if (plugin->priv->session != NULL)
-		return TRUE;
-
-	/* set up a session */
-	plugin->priv->session = soup_session_new_with_options (SOUP_SESSION_USER_AGENT,
-	                                                       "gnome-software",
-	                                                       NULL);
-	if (plugin->priv->session == NULL) {
-		g_set_error (error,
-			     GS_PLUGIN_ERROR,
-			     GS_PLUGIN_ERROR_FAILED,
-			     "%s: failed to setup networking",
-			     plugin->name);
-		return FALSE;
-	}
-	return TRUE;
-}
-
 /**
  * gs_plugin_app_set_rating_pkg:
  */
@@ -203,10 +168,15 @@ gs_plugin_app_set_rating_pkg (GsPlugin *plugin,
 		g_debug ("Failed to set rating on fedora-tagger: %s",
 			 soup_status_get_phrase (status_code));
 		if (msg->response_body->data != NULL) {
+			g_autoptr(GError) error_local = NULL;
 			error_msg = gs_plugin_parse_json (msg->response_body->data,
 							  msg->response_body->length,
-							  "error");
-			g_debug ("the error given was: %s", error_msg);
+							  "error",
+							  &error_local);
+			if (error_msg == NULL)
+				g_debug ("failed to parse fedora-tagger response: %s", error_local->message);
+			else
+				g_debug ("the error given was: %s", error_msg);
 		}
 	} else {
 		g_debug ("Got response: %s", msg->response_body->data);
@@ -235,10 +205,6 @@ gs_plugin_app_set_rating (GsPlugin *plugin,
 		return TRUE;
 	}
 
-	/* ensure networking is set up */
-	if (!gs_plugin_setup_networking (plugin, error))
-		return FALSE;
-
 	/* set rating for each package */
 	for (i = 0; i < sources->len; i++) {
 		pkgname = g_ptr_array_index (sources, i);
@@ -363,10 +329,6 @@ gs_plugin_fedora_tagger_download (GsPlugin *plugin, GError **error)
 			       GS_PLUGIN_FEDORA_TAGGER_SERVER);
 	msg = soup_message_new (SOUP_METHOD_GET, uri);
 
-	/* ensure networking is set up */
-	if (!gs_plugin_setup_networking (plugin, error))
-		return FALSE;
-
 	/* set sync request */
 	status_code = soup_session_send_message (plugin->priv->session, msg);
 	if (status_code != SOUP_STATUS_OK) {
@@ -477,7 +439,7 @@ gs_plugin_fedora_tagger_load_db (GsPlugin *plugin, GError **error)
 	/* create ratings if required */
 	rc = sqlite3_exec (plugin->priv->db,
 			   "SELECT vote_count FROM ratings LIMIT 1",
-			   gs_plugin_fedora_tagger_timestamp_cb, &mtime,
+			   NULL, NULL,
 			   &error_msg);
 	if (rc != SQLITE_OK) {
 		g_debug ("creating table to repair: %s", error_msg);
