@@ -29,6 +29,13 @@
 #include <gs-plugin.h>
 #include <gs-utils.h>
 
+/*
+ * SECTION:
+ * Gets the application usage data using the fedora-tagger service.
+ *
+ * It will self-disable if not run on a Fedora system.
+ */
+
 struct GsPluginPrivate {
 	SoupSession		*session;
 };
@@ -53,6 +60,8 @@ gs_plugin_initialize (GsPlugin *plugin)
 	g_autoptr(GSettings) settings = NULL;
 
 	plugin->priv = GS_PLUGIN_GET_PRIVATE (GsPluginPrivate);
+	plugin->priv->session = soup_session_new_with_options (SOUP_SESSION_USER_AGENT, gs_user_agent (),
+	                                                       NULL);
 
 	/* this is opt-in, and turned off by default */
 	settings = g_settings_new ("org.gnome.desktop.privacy");
@@ -93,31 +102,6 @@ gs_plugin_destroy (GsPlugin *plugin)
 		g_object_unref (plugin->priv->session);
 }
 
-/**
- * gs_plugin_setup_networking:
- */
-static gboolean
-gs_plugin_setup_networking (GsPlugin *plugin, GError **error)
-{
-	/* already set up */
-	if (plugin->priv->session != NULL)
-		return TRUE;
-
-	/* set up a session */
-	plugin->priv->session = soup_session_new_with_options (SOUP_SESSION_USER_AGENT,
-	                                                       "gnome-software",
-	                                                       NULL);
-	if (plugin->priv->session == NULL) {
-		g_set_error (error,
-			     GS_PLUGIN_ERROR,
-			     GS_PLUGIN_ERROR_FAILED,
-			     "%s: failed to setup networking",
-			     plugin->name);
-		return FALSE;
-	}
-	return TRUE;
-}
-
 /**
  * gs_plugin_app_set_usage_pkg:
  */
@@ -177,10 +161,6 @@ gs_plugin_app_set_usage_app (GsPlugin *plugin,
 	if (sources->len == 0)
 		return TRUE;
 
-	/* ensure networking is set up */
-	if (!gs_plugin_setup_networking (plugin, error))
-		return FALSE;
-
 	/* tell fedora-tagger about this package */
 	for (i = 0; i < sources->len; i++) {
 		pkgname = g_ptr_array_index (sources, i);
