@@ -31,8 +31,18 @@
 
 #include <gs-plugin.h>
 
+#include "gs-utils.h"
+
+/*
+ * SECTION:
+ * Queries for new firmware and schedules it to be installed as required.
+ *
+ * This plugin calls UpdatesChanged() if any updatable devices are
+ * added or removed or if a device has been updated live.
+ */
+
 struct GsPluginPrivate {
-	gsize			 done_init;
+	GMutex			 mutex;
 	GDBusProxy		*proxy;
 	GPtrArray		*to_download;
 	GPtrArray		*to_ignore;
@@ -40,39 +50,9 @@ struct GsPluginPrivate {
 	gchar			*cachedir;
 	gchar			*lvfs_sig_fn;
 	gchar			*lvfs_sig_hash;
+	gchar			*config_fn;
 };
 
-/**
- * gs_plugin_fwupd_setup_networking:
- */
-static gboolean
-gs_plugin_fwupd_setup_networking (GsPlugin *plugin, GError **error)
-{
-	g_autofree gchar *user_agent = NULL;
-
-	/* already set up */
-	if (plugin->priv->session != NULL)
-		return TRUE;
-
-	/* set up a session */
-	user_agent = g_strdup_printf ("%s/%s", PACKAGE_NAME, PACKAGE_VERSION);
-	plugin->priv->session = soup_session_new_with_options (SOUP_SESSION_USER_AGENT,
-	                                                       user_agent,
-	                                                       NULL);
-	if (plugin->priv->session == NULL) {
-		g_set_error (error,
-			     GS_PLUGIN_ERROR,
-			     GS_PLUGIN_ERROR_FAILED,
-			     "%s: failed to setup networking",
-			     plugin->name);
-		return FALSE;
-	}
-	/* this disables the double-compression of the firmware.xml.gz file */
-	soup_session_remove_feature_by_type (plugin->priv->session,
-					     SOUP_TYPE_CONTENT_DECODER);
-	return TRUE;
-}
-
 /**
  * gs_plugin_get_name:
  */
@@ -91,6 +71,21 @@ gs_plugin_initialize (GsPlugin *plugin)
 	plugin->priv = GS_PLUGIN_GET_PRIVATE (GsPluginPrivate);
 	plugin->priv->to_download = g_ptr_array_new_with_free_func (g_free);
 	plugin->priv->to_ignore = g_ptr_array_new_with_free_func (g_free);
+	plugin->priv->session = soup_session_new_with_options (SOUP_SESSION_USER_AGENT, gs_user_agent (),
+	                                                       NULL);
+	/* this disables the double-compression of the firmware.xml.gz file */
+	soup_session_remove_feature_by_type (plugin->priv->session,
+					     SOUP_TYPE_CONTENT_DECODER);
+
+	plugin->priv->config_fn = g_build_filename (SYSCONFDIR, "fwupd.conf", NULL);
+	if (!g_file_test (plugin->priv->config_fn, G_FILE_TEST_EXISTS)) {
+		g_free (plugin->priv->config_fn);
+		plugin->priv->config_fn = g_strdup ("/etc/fwupd.conf");
+	}
+	if (!g_file_test (plugin->priv->config_fn, G_FILE_TEST_EXISTS)) {
+		g_debug ("fwupd configuration not found, disabling plugin.");
+		gs_plugin_set_enabled (plugin, FALSE);
+	}
 }
 
 /**
@@ -102,6 +97,7 @@ gs_plugin_destroy (GsPlugin *plugin)
 	g_free (plugin->priv->cachedir);
 	g_free (plugin->priv->lvfs_sig_fn);
 	g_free (plugin->priv->lvfs_sig_hash);
+	g_free (plugin->priv->config_fn);
 	g_ptr_array_unref (plugin->priv->to_download);
 	g_ptr_array_unref (plugin->priv->to_ignore);
 	if (plugin->priv->proxy != NULL)
@@ -135,6 +131,7 @@ gs_plugin_startup (GsPlugin *plugin, GCancellable *cancellable, GError **error)
 	g_autoptr(GError) error_local = NULL;
 	g_autofree gchar *data = NULL;
 	g_autoptr(GDBusConnection) conn = NULL;
+	g_autoptr(GMutexLocker) locker = g_mutex_locker_new (&plugin->priv->mutex);
 
 	/* register D-Bus errors */
 	fwupd_error_quark ();
@@ -231,46 +228,18 @@ gs_plugin_fwupd_set_app_from_kv (GsApp *app, const gchar *key, GVariant *val)
 {
 	g_debug ("key %s", key);
 
-	if (g_strcmp0 (key, "Guid") == 0) {
+	if (g_strcmp0 (key, "AppstreamId") == 0) {
 		gs_app_set_id (app, g_variant_get_string (val, NULL));
 		return;
 	}
+	if (g_strcmp0 (key, "Guid") == 0) {
+		gs_app_set_metadata (app, "GUID", g_variant_get_string (val, NULL));
+		return;
+	}
 	if (g_strcmp0 (key, "Version") == 0) {
 		gs_app_set_version (app, g_variant_get_string (val, NULL));
 		return;
 	}
-	if (g_strcmp0 (key, "Vendor") == 0) {
-		gs_app_set_origin (app, g_variant_get_string (val, NULL));
-		return;
-	}
-	if (g_strcmp0 (key, "Name") == 0) {
-		gs_app_add_source (app, g_variant_get_string (val, NULL));
-		gs_app_set_name (app, GS_APP_QUALITY_NORMAL,
-				 g_variant_get_string (val, NULL));
-		return;
-	}
-	if (g_strcmp0 (key, "Summary") == 0) {
-		gs_app_set_summary (app, GS_APP_QUALITY_NORMAL,
-				    g_variant_get_string (val, NULL));
-		return;
-	}
-	if (g_strcmp0 (key, "Description") == 0) {
-		g_autofree gchar *tmp = NULL;
-		tmp = as_markup_convert (g_variant_get_string (val, NULL),
-					 AS_MARKUP_CONVERT_FORMAT_SIMPLE, NULL);
-		if (tmp != NULL)
-			gs_app_set_description (app, GS_APP_QUALITY_HIGHEST, tmp);
-		return;
-	}
-	if (g_strcmp0 (key, "UrlHomepage") == 0) {
-		gs_app_set_url (app, AS_URL_KIND_HOMEPAGE,
-				g_variant_get_string (val, NULL));
-		return;
-	}
-	if (g_strcmp0 (key, "License") == 0) {
-		gs_app_set_licence (app, g_variant_get_string (val, NULL));
-		return;
-	}
 	if (g_strcmp0 (key, "Size") == 0) {
 		gs_app_set_size (app, g_variant_get_uint64 (val));
 		return;
@@ -279,6 +248,12 @@ gs_plugin_fwupd_set_app_from_kv (GsApp *app, const gchar *key, GVariant *val)
 		gs_app_set_update_version (app, g_variant_get_string (val, NULL));
 		return;
 	}
+	if (g_strcmp0 (key, "License") == 0) {
+		gs_app_set_licence (app,
+				    g_variant_get_string (val, NULL),
+				    GS_APP_QUALITY_NORMAL);
+		return;
+	}
 	if (g_strcmp0 (key, "UpdateDescription") == 0) {
 		g_autofree gchar *tmp = NULL;
 		tmp = as_markup_convert (g_variant_get_string (val, NULL),
@@ -394,15 +369,20 @@ gs_plugin_add_update_app (GsPlugin *plugin,
 		return FALSE;
 	}
 
+	/* can be done live */
+	if (flags & FU_DEVICE_FLAG_ALLOW_ONLINE) {
+		gs_app_set_state (app, AS_APP_STATE_UPDATABLE_LIVE);
+	} else {
+		gs_app_set_state (app, AS_APP_STATE_UPDATABLE);
+	}
+
 	/* actually add the application */
 	gs_app_set_management_plugin (app, "fwupd");
-	gs_app_set_state (app, AS_APP_STATE_UPDATABLE);
 	gs_app_set_id_kind (app, AS_ID_KIND_FIRMWARE);
 	gs_app_add_source_id (app, filename_cache);
 	gs_app_add_category (app, "System");
-	gs_app_set_kind (app, GS_APP_KIND_SYSTEM);
+	gs_app_set_kind (app, GS_APP_KIND_FIRMWARE_UPDATE);
 	gs_app_set_metadata (app, "fwupd::DeviceID", id);
-	gs_app_set_metadata (app, "DataDir::desktop-icon", "application-x-firmware");
 	gs_plugin_add_app (list, app);
 
 	/* create icon */
@@ -425,23 +405,20 @@ gs_plugin_add_updates_historical (GsPlugin *plugin,
 {
 	GVariant *variant;
 	const gchar *key;
-	gboolean ret;
 	g_autoptr(GError) error_local = NULL;
 	g_autoptr(GsApp) app = NULL;
 	g_autoptr(GVariantIter) iter = NULL;
 	g_autoptr(GVariant) val = NULL;
 
-	/* watch the file in case it comes or goes */
-	if (g_once_init_enter (&plugin->priv->done_init)) {
-		ret = gs_plugin_startup (plugin, cancellable, error);
-		g_once_init_leave (&plugin->priv->done_init, TRUE);
-		if (!ret)
+	/* set up plugin */
+	if (plugin->priv->proxy == NULL) {
+		if (!gs_plugin_startup (plugin, cancellable, error))
 			return FALSE;
 	}
-
-	/* could not connect */
 	if (plugin->priv->proxy == NULL)
 		return TRUE;
+
+	/* get historical updates */
 	val = g_dbus_proxy_call_sync (plugin->priv->proxy,
 				      "GetResults",
 				      g_variant_new ("(s)", FWUPD_DEVICE_ID_ANY),
@@ -450,6 +427,13 @@ gs_plugin_add_updates_historical (GsPlugin *plugin,
 				      NULL,
 				      &error_local);
 	if (val == NULL) {
+		if (g_error_matches (error_local,
+				     G_DBUS_ERROR,
+				     G_DBUS_ERROR_SERVICE_UNKNOWN)) {
+			/* the fwupd service might be unavailable, continue in that case */
+			g_debug ("fwupd: Could not get historical updates, service is unknown.");
+			return TRUE;
+		}
 		if (g_error_matches (error_local,
 				     FWUPD_ERROR,
 				     FWUPD_ERROR_NOTHING_TO_DO))
@@ -490,23 +474,20 @@ gs_plugin_add_updates (GsPlugin *plugin,
 		       GError **error)
 {
 	const gchar *id;
-	gboolean ret;
 	GVariantIter *iter_device;
 	g_autoptr(GError) error_local = NULL;
 	g_autoptr(GVariantIter) iter = NULL;
 	g_autoptr(GVariant) val = NULL;
 
-	/* watch the file in case it comes or goes */
-	if (g_once_init_enter (&plugin->priv->done_init)) {
-		ret = gs_plugin_startup (plugin, cancellable, error);
-		g_once_init_leave (&plugin->priv->done_init, TRUE);
-		if (!ret)
+	/* set up plugin */
+	if (plugin->priv->proxy == NULL) {
+		if (!gs_plugin_startup (plugin, cancellable, error))
 			return FALSE;
 	}
-
-	/* could not connect */
 	if (plugin->priv->proxy == NULL)
 		return TRUE;
+
+	/* get current list of updates */
 	val = g_dbus_proxy_call_sync (plugin->priv->proxy,
 				      "GetUpdates",
 				      NULL,
@@ -515,6 +496,13 @@ gs_plugin_add_updates (GsPlugin *plugin,
 				      NULL,
 				      &error_local);
 	if (val == NULL) {
+		if (g_error_matches (error_local,
+				     G_DBUS_ERROR,
+				     G_DBUS_ERROR_SERVICE_UNKNOWN)) {
+			/* the fwupd service might be unavailable, continue in that case */
+			g_debug ("fwupd: Could not get updates, service is unknown.");
+			return TRUE;
+		}
 		if (g_error_matches (error_local,
 				     FWUPD_ERROR,
 				     FWUPD_ERROR_NOTHING_TO_DO))
@@ -617,32 +605,6 @@ gs_plugin_fwupd_update_lvfs_metadata (const gchar *data_fn, const gchar *sig_fn,
 	return TRUE;
 }
 
-/**
- * gs_plugin_fwupd_get_file_cache_age:
- */
-static guint
-gs_plugin_fwupd_get_file_cache_age (const gchar *fn)
-{
-	guint64 now;
-	guint64 mtime;
-	g_autoptr(GFile) file = NULL;
-	g_autoptr(GFileInfo) info = NULL;
-
-	file = g_file_new_for_path (fn);
-	info = g_file_query_info (file,
-				  G_FILE_ATTRIBUTE_TIME_MODIFIED,
-				  G_FILE_QUERY_INFO_NONE,
-				  NULL,
-				  NULL);
-	if (info == NULL)
-		return G_MAXUINT;
-	mtime = g_file_info_get_attribute_uint64 (info, G_FILE_ATTRIBUTE_TIME_MODIFIED);
-	now = (guint64) g_get_real_time () / G_USEC_PER_SEC;
-	if (mtime > now)
-		return G_MAXUINT;
-	return now - mtime;
-}
-
 /**
  * gs_plugin_fwupd_check_lvfs_metadata:
  */
@@ -657,7 +619,6 @@ gs_plugin_fwupd_check_lvfs_metadata (GsPlugin *plugin,
 	g_autofree gchar *basename_data = NULL;
 	g_autofree gchar *cache_fn_data = NULL;
 	g_autofree gchar *checksum = NULL;
-	g_autofree gchar *config_fn = NULL;
 	g_autofree gchar *url_data = NULL;
 	g_autofree gchar *url_sig = NULL;
 	g_autoptr(GKeyFile) config = NULL;
@@ -666,18 +627,13 @@ gs_plugin_fwupd_check_lvfs_metadata (GsPlugin *plugin,
 
 	/* read config file */
 	config = g_key_file_new ();
-	config_fn = g_build_filename (SYSCONFDIR, "fwupd.conf", NULL);
-	if (!g_file_test (config_fn, G_FILE_TEST_EXISTS)) {
-		g_free (config_fn);
-		config_fn = g_strdup ("/etc/fwupd.conf");
-	}
-	if (!g_key_file_load_from_file (config, config_fn, G_KEY_FILE_NONE, error))
+	if (!g_key_file_load_from_file (config, plugin->priv->config_fn, G_KEY_FILE_NONE, error))
 		return FALSE;
 
 	/* check cache age */
 	if (cache_age > 0) {
 		guint tmp;
-		tmp = gs_plugin_fwupd_get_file_cache_age (plugin->priv->lvfs_sig_fn);
+		tmp = gs_utils_get_file_age (plugin->priv->lvfs_sig_fn);
 		if (tmp < cache_age) {
 			g_debug ("%s is only %i seconds old, so ignoring refresh",
 				 plugin->priv->lvfs_sig_fn, tmp);
@@ -772,20 +728,15 @@ gs_plugin_refresh (GsPlugin *plugin,
 		   GError **error)
 {
 	const gchar *tmp;
-	gboolean ret;
 	guint i;
 
 	/* set up plugin */
-	if (g_once_init_enter (&plugin->priv->done_init)) {
-		ret = gs_plugin_startup (plugin, cancellable, error);
-		g_once_init_leave (&plugin->priv->done_init, TRUE);
-		if (!ret)
+	if (plugin->priv->proxy == NULL) {
+		if (!gs_plugin_startup (plugin, cancellable, error))
 			return FALSE;
 	}
-
-	/* ensure networking is set up */
-	if (!gs_plugin_fwupd_setup_networking (plugin, error))
-		return FALSE;
+	if (plugin->priv->proxy == NULL)
+		return TRUE;
 
 	/* get the metadata and signature file */
 	if (!gs_plugin_fwupd_check_lvfs_metadata (plugin, cache_age, cancellable, error))
@@ -810,7 +761,7 @@ gs_plugin_refresh (GsPlugin *plugin,
 		if (status_code != SOUP_STATUS_OK) {
 			g_warning ("Failed to download %s, ignoring: %s",
 				   tmp, soup_status_get_phrase (status_code));
-			g_ptr_array_remove (plugin->priv->to_download, (gpointer) tmp);
+			g_ptr_array_remove_index (plugin->priv->to_download, i--);
 			g_ptr_array_add (plugin->priv->to_ignore, g_strdup (tmp));
 			continue;
 		}
@@ -839,6 +790,7 @@ static gboolean
 gs_plugin_fwupd_upgrade (GsPlugin *plugin,
 			 const gchar *filename,
 			 const gchar *device_id,
+			 gboolean do_offline,
 			 GCancellable *cancellable,
 			 GError **error)
 {
@@ -861,8 +813,10 @@ gs_plugin_fwupd_upgrade (GsPlugin *plugin,
 			       "reason", g_variant_new_string ("system-update"));
 	g_variant_builder_add (&builder, "{sv}",
 			       "filename", g_variant_new_string (filename));
-	g_variant_builder_add (&builder, "{sv}",
-			       "offline", g_variant_new_boolean (TRUE));
+	if (do_offline) {
+		g_variant_builder_add (&builder, "{sv}",
+				       "offline", g_variant_new_boolean (TRUE));
+	}
 
 	/* open file */
 	fd = open (filename, O_RDONLY);
@@ -937,7 +891,7 @@ gs_plugin_app_upgrade (GsPlugin *plugin,
 		return FALSE;
 	}
 	gs_app_set_state (app, AS_APP_STATE_INSTALLING);
-	if (!gs_plugin_fwupd_upgrade (plugin, filename, device_id,
+	if (!gs_plugin_fwupd_upgrade (plugin, filename, device_id, TRUE,
 				      cancellable, error))
 		return FALSE;
 	gs_app_set_state (app, AS_APP_STATE_INSTALLED);
@@ -966,6 +920,8 @@ gs_plugin_offline_update (GsPlugin *plugin,
 
 /**
  * gs_plugin_app_install:
+ *
+ * Called when a user double clicks on a .cab file
  */
 gboolean
 gs_plugin_app_install (GsPlugin *plugin,
@@ -974,6 +930,7 @@ gs_plugin_app_install (GsPlugin *plugin,
 		       GError **error)
 {
 	const gchar *filename;
+	gboolean offline = TRUE;
 
 	/* only process this app if was created by this plugin */
 	if (g_strcmp0 (gs_app_get_management_plugin (app), "fwupd") != 0)
@@ -989,13 +946,29 @@ gs_plugin_app_install (GsPlugin *plugin,
 		return FALSE;
 	}
 	gs_app_set_state (app, AS_APP_STATE_INSTALLING);
-	if (!gs_plugin_fwupd_upgrade (plugin, filename, FWUPD_DEVICE_ID_ANY,
+	if (gs_app_get_kind (app) == GS_APP_KIND_FIRMWARE_UPDATE)
+		offline = FALSE;
+	if (!gs_plugin_fwupd_upgrade (plugin, filename, FWUPD_DEVICE_ID_ANY, offline,
 				      cancellable, error))
 		return FALSE;
 	gs_app_set_state (app, AS_APP_STATE_INSTALLED);
 	return TRUE;
 }
 
+/**
+ * gs_plugin_app_update:
+ *
+ * This is only called when updating device firmware live.
+ */
+gboolean
+gs_plugin_app_update (GsPlugin *plugin,
+		      GsApp *app,
+		      GCancellable *cancellable,
+		      GError **error)
+{
+	return gs_plugin_app_install (plugin, app, cancellable, error);
+}
+
 /**
  * gs_plugin_fwupd_content_type_matches:
  */
@@ -1119,10 +1092,9 @@ gs_plugin_filename_to_app (GsPlugin *plugin,
 
 	/* get results */
 	app = gs_app_new (NULL);
-	gs_app_set_metadata (app, "DataDir::desktop-icon", "application-x-firmware");
 	gs_app_set_id_kind (app, AS_ID_KIND_FIRMWARE);
 	gs_app_set_management_plugin (app, "fwupd");
-	gs_app_set_kind (app, GS_APP_KIND_SYSTEM);
+	gs_app_set_kind (app, GS_APP_KIND_FIRMWARE_UPDATE);
 	gs_app_set_state (app, AS_APP_STATE_AVAILABLE_LOCAL);
 	gs_app_add_source_id (app, filename);
 	gs_app_add_category (app, "System");
