@@ -27,8 +27,17 @@
 #include <gs-plugin.h>
 #include <gs-utils.h>
 
+/*
+ * SECTION:
+ * Provides a local SQL database of user-set ratings, useful for
+ * testing or when the user 'overrides' the default rating.
+ *
+ * Methods:     | AppSetRating
+ * Refines:     | [id]->[rating]
+ */
+
 struct GsPluginPrivate {
-	gsize                    loaded;
+	gsize			 loaded;
 	gchar			*db_path;
 	sqlite3			*db;
 };
@@ -50,9 +59,7 @@ gs_plugin_initialize (GsPlugin *plugin)
 {
 	/* create private area */
 	plugin->priv = GS_PLUGIN_GET_PRIVATE (GsPluginPrivate);
-	plugin->priv->db_path = g_build_filename (g_get_home_dir (),
-						  ".local",
-						  "share",
+	plugin->priv->db_path = g_build_filename (g_get_user_data_dir (),
 						  "gnome-software",
 						  "hardcoded-ratings.db",
 						  NULL);
